@@ -9,10 +9,11 @@ GVWebPage::GVWebPage(QObject *parent/* = NULL*/)
 , bUseIphoneUA (true)
 , webPage (this)
 , garbageTimer (this)
+, nwCfg (this)
+, pageTimeoutTimer (this)
+, pCurrentReply (NULL)
 {
     webPage.settings()->setAttribute (QWebSettings::JavaEnabled   , false);
-//     webPage.settings()->setAttribute (QWebSettings::PluginsEnabled, false);
-//     webPage.settings()->setAttribute (QWebSettings::AutoLoadImages, false);
     webPage.setForwardUnsupportedContent (true);
 
     garbageTimer.setSingleShot (true);
@@ -30,6 +31,10 @@ GVWebPage::GVWebPage(QObject *parent/* = NULL*/)
     QObject::connect (&garbageTimer, SIGNAL (timeout ()),
                        this        , SLOT   (garbageTimerTimeout ()));
     garbageTimer.start ();
+
+    // Page timeout timer
+    QObject::connect (&pageTimeoutTimer, SIGNAL (timeout()),
+                       this            , SLOT   (onPageTimeout()));
 }//GVWebPage::GVWebPage
 
 GVWebPage::~GVWebPage(void)
@@ -61,6 +66,7 @@ void
 GVWebPage::loadUrlString (const QString &strUrl)
 {
     webPage.mainFrame()->load (QUrl (strUrl));
+    onPageProgress (0);
     if (NULL != webPage.view ())
     {
         webPage.view()->show ();
@@ -141,6 +147,12 @@ GVWebPage::aboutBlankDone (bool bOk)
 bool
 GVWebPage::login ()
 {
+    if (!nwCfg.isOnline ()) {
+        qDebug ("Cannot login when offline");
+        completeCurrentWork (GVAW_login, false);
+        return false;
+    }
+
     webPage.setUA (bUseIphoneUA);
 
     // GV page load complete will begin the login process.
@@ -329,11 +341,15 @@ GVWebPage::loginStage3 (bool bOk)
 bool
 GVWebPage::logout ()
 {
-    QString strLink = GV_HTTPS "/account/signout";
+    if (!nwCfg.isOnline ()) {
+        qDebug ("Cannot logout when offline");
+        completeCurrentWork (GVAW_logout, false);
+        return false;
+    }
 
     QObject::connect (&webPage, SIGNAL (loadFinished (bool)),
                        this   , SLOT   (logoutDone (bool)));
-    this->loadUrlString (strLink);
+    this->loadUrlString (GV_HTTPS "/account/signout");
 
     return (true);
 }//GVWebPage::logout
@@ -356,6 +372,12 @@ GVWebPage::logoutDone (bool bOk)
 bool
 GVWebPage::retrieveContacts ()
 {
+    if (!nwCfg.isOnline ()) {
+        qDebug ("Cannot retrieve contacts when offline");
+        completeCurrentWork (GVAW_getAllContacts, false);
+        return false;
+    }
+
     QMutexLocker locker(&mutex);
     if (!bLoggedIn)
     {
@@ -458,6 +480,12 @@ GVWebPage::isNextContactsPageAvailable ()
 bool
 GVWebPage::dialCallback (bool bCallback)
 {
+    if (!nwCfg.isOnline ()) {
+        qDebug ("Cannot dial back when offline");
+        completeCurrentWork (bCallback?GVAW_dialCallback:GVAW_dialOut, false);
+        return false;
+    }
+
     QMutexLocker locker(&mutex);
     if (!bLoggedIn)
     {
@@ -468,6 +496,7 @@ GVWebPage::dialCallback (bool bCallback)
     QVariantList &arrParams = workCurrent.arrParams;
     QStringPairList arrPairs;
     workCurrent.cancel = (WebPageCancel) &GVWebPage::cancelDataDial2;
+    QNetworkReply *reply = NULL;
 
     if (!bCallback)
     {
@@ -533,7 +562,7 @@ GVWebPage::dialCallback (bool bCallback)
 
         QObject::connect (mgr , SIGNAL (finished (QNetworkReply *)),
                           this, SLOT   (onDataCallDone (QNetworkReply *)));
-        mgr->post (request, strContent.toAscii());
+        reply = mgr->post (request, strContent.toAscii());
     }
     else
     {
@@ -543,10 +572,13 @@ GVWebPage::dialCallback (bool bCallback)
         arrPairs += QStringPair("phoneType"       , arrParams[2].toString());
         arrPairs += QStringPair("remember"        , "1");
         arrPairs += QStringPair("_rnr_se"         , strRnr_se);
+        reply =
         postRequest (GV_DATA_BASE "/call/connect/", arrPairs, UA_IPHONE,
                      this, SLOT (onDataCallDone (QNetworkReply *)));
     }
 
+    startTimerForReply (reply);
+
     return (true);
 }//GVWebPage::dialCallback
 
@@ -561,6 +593,7 @@ GVWebPage::onDataCallDone (QNetworkReply * reply)
 
     bool bOk = false;
     do { // Begin cleanup block (not a loop)
+        qDebug () << msg;
         QRegExp rx("\"access_number\":\"([+\\d]*)\"");
         if (msg.contains (rx) && (1 == rx.captureCount ()))
         {
@@ -634,6 +667,12 @@ GVWebPage::onDataCallCanceled (QNetworkReply * reply)
 bool
 GVWebPage::getContactInfoFromLink ()
 {
+    if (!nwCfg.isOnline ()) {
+        qDebug ("Cannot retrieve contact info when offline");
+        completeCurrentWork (GVAW_getContactFromLink, false);
+        return false;
+    }
+
     QMutexLocker locker(&mutex);
     if (!bLoggedIn)
     {
@@ -729,6 +768,12 @@ GVWebPage::contactInfoLoaded (bool bOk)
 bool
 GVWebPage::getRegisteredPhones ()
 {
+    if (!nwCfg.isOnline ()) {
+        qDebug ("Cannot get registered phones when offline");
+        completeCurrentWork (GVAW_getRegisteredPhones, false);
+        return false;
+    }
+
     QMutexLocker locker(&mutex);
     if (!bLoggedIn)
     {
@@ -811,6 +856,12 @@ GVWebPage::userCancel ()
 bool
 GVWebPage::sendInboxRequest ()
 {
+    if (!nwCfg.isOnline ()) {
+        qDebug ("Cannot send request for inbox when offline");
+        completeCurrentWork (GVAW_getInbox, false);
+        return false;
+    }
+
     QMutexLocker locker(&mutex);
     if (!bLoggedIn)
     {
@@ -869,6 +920,12 @@ GVWebPage::sendInboxRequest ()
 bool
 GVWebPage::getInbox ()
 {
+    if (!nwCfg.isOnline ()) {
+        qDebug ("Cannot get inbox when offline");
+        completeCurrentWork (GVAW_getInbox, false);
+        return false;
+    }
+
     QMutexLocker locker(&mutex);
     if (!bLoggedIn)
     {
@@ -947,6 +1004,12 @@ GVWebPage::onGotInboxXML (QNetworkReply *reply)
 bool
 GVWebPage::getContactFromInboxLink ()
 {
+    if (!nwCfg.isOnline ()) {
+        qDebug ("Cannot get contact from inbox link when offline");
+        completeCurrentWork (GVAW_getContactFromInboxLink, false);
+        return false;
+    }
+
     QMutexLocker locker(&mutex);
     if (!bLoggedIn)
     {
@@ -1048,6 +1111,12 @@ GVWebPage::garbageTimerTimeout ()
 bool
 GVWebPage::sendSMS ()
 {
+    if (!nwCfg.isOnline ()) {
+        qDebug ("Cannot send SMS when offline");
+        completeCurrentWork (GVAW_sendSMS, false);
+        return false;
+    }
+
     QMutexLocker locker(&mutex);
     if (!bLoggedIn)
     {
@@ -1142,6 +1211,12 @@ GVWebPage::sendSMSResponse (QNetworkReply *reply)
 bool
 GVWebPage::playVmail ()
 {
+    if (!nwCfg.isOnline ()) {
+        qDebug ("Cannot download vmail when offline");
+        completeCurrentWork (GVAW_playVmail, false);
+        return false;
+    }
+
     QMutexLocker locker(&mutex);
     if (!bLoggedIn)
     {
@@ -1223,3 +1298,74 @@ GVWebPage::onVmailDownloaded (QNetworkReply *reply)
     completeCurrentWork (GVAW_playVmail, rv);
     reply->deleteLater ();
 }//GVWebPage::onVmailDownloaded
+
+void
+GVWebPage::onPageTimeout ()
+{
+    if (NULL != pCurrentReply) {
+        qWarning ("Request has timed out. Aborting!!!");
+        pCurrentReply->abort ();
+
+        QObject::disconnect (
+            pCurrentReply, SIGNAL(downloadProgress(qint64,qint64)),
+            this         , SLOT(onSocketXfer(qint64,qint64)));
+        QObject::disconnect (
+            pCurrentReply, SIGNAL(uploadProgress(qint64,qint64)),
+            this         , SLOT(onSocketXfer(qint64,qint64)));
+        pCurrentReply = NULL;
+
+        //@@UV: Test if this is required
+        cancelWork ();
+    } else {
+        qWarning ("Web page load has timed out. Aborting!!!");
+        userCancel ();
+    }
+}//GVWebPage::onPageTimeout
+
+void
+GVWebPage::onPageProgress(int /*progress*/)
+{
+    pageTimeoutTimer.stop ();
+    qDebug("Page progressed. Not timing out!");
+    pageTimeoutTimer.setInterval (45 * 1000);
+    pageTimeoutTimer.setSingleShot (true);
+    pageTimeoutTimer.start ();
+}//GVWebPage::onPageProgress
+
+void
+GVWebPage::onSocketXfer (qint64 /*bytesXfer*/, qint64 /*bytesTotal*/)
+{
+    pageTimeoutTimer.stop ();
+    qDebug("Socket transferred data. Not timing out!");
+    pageTimeoutTimer.setInterval (15 * 1000);
+    pageTimeoutTimer.setSingleShot (true);
+    pageTimeoutTimer.start ();
+}//GVWebPage::onSocketXfer
+
+void
+GVWebPage::completeCurrentWork (GVAccess_Work whatwork, bool bOk)
+{
+    pageTimeoutTimer.stop ();
+    if (NULL != pCurrentReply) {
+        QObject::disconnect (
+            pCurrentReply, SIGNAL(downloadProgress(qint64,qint64)),
+            this         , SLOT(onSocketXfer(qint64,qint64)));
+        QObject::disconnect (
+            pCurrentReply, SIGNAL(uploadProgress(qint64,qint64)),
+            this         , SLOT(onSocketXfer(qint64,qint64)));
+        pCurrentReply = NULL;
+    }
+
+    GVAccess::completeCurrentWork (whatwork, bOk);
+}//GVWebPage::completeCurrentWork
+
+void
+GVWebPage::startTimerForReply (QNetworkReply *reply)
+{
+    pCurrentReply = reply;
+    QObject::connect (reply, SIGNAL(downloadProgress(qint64,qint64)),
+                      this , SLOT(onSocketXfer(qint64,qint64)));
+    QObject::connect (reply, SIGNAL(uploadProgress(qint64,qint64)),
+                      this , SLOT(onSocketXfer(qint64,qint64)));
+    onSocketXfer (0,0);
+}//GVWebPage::startTimerForReply
