@@ -374,6 +374,8 @@ MainWindow::initQML ()
                       this, SLOT   (doLogin ()));
     QObject::connect (gObj, SIGNAL (sigLogout ()),
                       this, SLOT   (doLogout ()));
+    QObject::connect (gObj, SIGNAL (sigWebPage ()),
+                      this, SLOT   (on_actionWeb_view_triggered ()));
     QObject::connect (gObj, SIGNAL (sigRefresh ()),
                       this, SLOT   (onRefresh ()));
     QObject::connect (gObj, SIGNAL (sigRefreshAll ()),
@@ -892,7 +894,6 @@ MainWindow::contactsLinkWorkDone (bool, const QVariantList &)
 void
 MainWindow::dialNow (const QString &strTarget)
 {
-    bool bDialout;
     CalloutInitiator *ci;
 
     do // Begin cleanup block (not a loop)
@@ -1054,9 +1055,13 @@ MainWindow::onDialDlgClose (int retval, const QString & /*strNumber*/)
     }
     else
     {
-        bDialCancelled = true;
         GVAccess &webPage = Singletons::getRef().getGVAccess ();
-        webPage.cancelWork (GVAW_dialCallback);
+        bDialCancelled = true;
+        if (bDialout) {
+            webPage.cancelWork (GVAW_dialOut);
+        } else {
+            webPage.cancelWork (GVAW_dialCallback);
+        }
     }
 }//MainWindow::onDialDlgClose
 
